/*! 
    \file particle_system.h

*/

#ifndef _PARTICLE_SYSTEM_H
#define _PARTICLE_SYSTEM_H

#include "base_object.h"
#include "vector.h"

//! An enumerator for the different types of particles.
typedef enum PARTICLE_TYPE {PARTICLE_DOT,
			    PARTICLE_SPRITE,
			    PARTICLE_OBJECT,
			    PARTICLE_OBJECT_ARRAY,
			    PARTICLE_PRIMITIVE};

#define PARTICLE_DEFAULT_COUNT 200             //!< a default count of particles in the system.
#define PARTICLE_DEFAULT_TYPE  PARTICLE_SPRITE //!< A default type of the system.

// FORWARD DEFINITION
class Material;
class ParticleEmitter;


//! A struct for one Particle
typedef struct Particle
{
  float timeToLive;           //!< The time this particle lives from NOW on.
  Vector position;            //!< The current position of this particle.
  Vector velocity;            //!< The current velocity of this particle.
  Quaternion rotation;        //!< The current rotation of this particle.

  //  Particle* next;            //!< pointer to the next particle in the List. (NULL if no preceding one)
};

//! A class to handle particle Systems
class ParticleSystem : public BaseObject {

 public:
  ParticleSystem(unsigned int particleCount = PARTICLE_DEFAULT_COUNT, PARTICLE_TYPE type = PARTICLE_DEFAULT_TYPE);
  virtual ~ParticleSystem();

  void setMaterial(Material* material);
  void addEmitter(ParticleEmitter* emitter);

 private:
  int particleCount;         //!< the count of Particles for this ParticleSystem.
  PARTICLE_TYPE particleType;//!< A type for the Particles
  Particle* particles;       //!< A list of particles of this System.
  Material* material;        //!< A Material for all the Particles.

  ParticleEmitter* emitter;  //!< An emitter for this particleSystem.
};

#endif /* _PARTICLE_SYSTEM_H */
